@@ -7,6 +7,7 @@ import asyncio
 import os
 import shutil
 import socket
+import stat
 import sys
 import time
 
@@ -158,6 +159,8 @@ class WSTerminalClient(object):
         self._workspace = None
         self._running = False
         self._download_file = {}
+        self._writing_files = {}
+        utils.safe_ensure_future(self.write_file_task())
 
     @property
     def auto_reconnect(self):
@@ -199,8 +202,10 @@ class WSTerminalClient(object):
             path = (root + "/" + name) if root else name
             if not dir_tree["dirs"][name]:
                 # Blank directory
+                utils.write_stdout_inplace("Create directory %s" % path)
                 await self.create_directory(path)
             elif dir_tree["dirs"][name] == "-":
+                utils.write_stdout_inplace("Remove directory %s" % path)
                 await self.remove_directory(path)
             else:
                 await self.update_workspace(
@@ -210,33 +215,52 @@ class WSTerminalClient(object):
             path = (root + "/" + name) if root else name
             if dir_tree["files"][name] == "-":
                 # Remove file
+                utils.write_stdout_inplace("Remove file %s" % path)
                 await self.remove_file(path)
             else:
+                utils.write_stdout_inplace("Sync file %s" % path)
                 await self.write_file(path)
+                await self.set_perm(path)
 
     async def write_file(self, file_path):
         utils.logger.debug("[%s] Write file %s" % (self.__class__.__name__, file_path))
-        offset = 0
-        try:
-            with open(self._workspace.join_path(file_path), "rb") as fp:
-                while True:
-                    data = fp.read(self.file_fragment_size)
-                    if not data and offset:
-                        break
-                    await self._conn.send_request(
-                        proto.EnumCommand.WRITE_FILE,
-                        path=file_path.replace(os.path.sep, "/"),
-                        data=data,
-                        overwrite=offset == 0,
-                    )
-                    if not data:
-                        break
-                    offset += len(data)
-        except FileNotFoundError:
+        abs_file_path = self._workspace.join_path(file_path)
+        if not os.path.isfile(abs_file_path):
             utils.logger.warn(
                 "[%s] File %s removed before read"
                 % (self.__class__.__name__, file_path)
             )
+            return
+        offset = 0
+        with open(abs_file_path, "rb") as fp:
+            while True:
+                data = fp.read(self.file_fragment_size)
+                if not data and offset:
+                    break
+                await self._conn.send_request(
+                    proto.EnumCommand.WRITE_FILE,
+                    path=file_path.replace(os.path.sep, "/"),
+                    data=data,
+                    overwrite=offset == 0,
+                )
+                if not data:
+                    break
+                offset += len(data)
+
+    async def delay_write_file(self, file_path, delay_time):
+        if file_path not in self._writing_files:
+            self._writing_files[file_path] = time.time() + delay_time
+
+    async def write_file_task(self):
+        while self._running:
+            if not self._writing_files:
+                await asyncio.sleep(0.1)
+                continue
+            for file_path in list(self._writing_files.keys()):
+                if time.time() >= self._writing_files[file_path]:
+                    await self.write_file(file_path)
+                    self._writing_files.pop(file_path)
+            await asyncio.sleep(0.01)
 
     async def remove_file(self, file_path):
         utils.logger.info("[%s] Remove file %s" % (self.__class__.__name__, file_path))
@@ -254,6 +278,15 @@ class WSTerminalClient(object):
             dst_path=dst_path.replace(os.path.sep, "/"),
         )
 
+    async def set_perm(self, path):
+        utils.logger.info("[%s] Set permission of %s" % (self.__class__.__name__, path))
+        abs_path = self._workspace.join_path(path)
+        permission = os.stat(abs_path).st_mode & 0o777
+        if permission & stat.S_IXOTH:
+            await self._conn.send_request(
+                proto.EnumCommand.SET_PERM, path=path, perm=permission
+            )
+
     async def on_directory_created(self, path):
         utils.logger.debug(
             "[%s] New directory %s created" % (self.__class__.__name__, path)
@@ -271,7 +304,7 @@ class WSTerminalClient(object):
 
     async def on_file_modified(self, path):
         utils.logger.debug("[%s] File %s modified" % (self.__class__.__name__, path))
-        await self.write_file(path)
+        await self.delay_write_file(path, 0.5)
 
     async def on_file_removed(self, path):
         utils.logger.debug("[%s] File %s removed" % (self.__class__.__name__, path))
@@ -302,9 +335,11 @@ class WSTerminalClient(object):
         response = await self._conn.read_response(request)
         if response["code"] != 0:
             raise utils.WSTermRuntimeError(response["code"], response["message"])
+        utils.write_stdout_inplace("Create workspace diff list")
         diff_result = self._workspace.make_diff(response["data"])
         if diff_result:
             await self.update_workspace(diff_result, "")
+        utils.write_stdout_inplace("\n")
         self._workspace.register_handler(self)
         asyncio.ensure_future(self._workspace.watch())
 
